#!/usr/bin/perl 
#===============================================================================
#
#         FILE: detectdcdv3.pl
#
#        USAGE: ./detectdcdv3.pl
#
#  DESCRIPTION:
#
#      OPTIONS: perl detectdcdv4_web.pl chainId
# REQUIREMENTS: ---
#         BUGS: ---
#        NOTES: ---
#       AUTHOR: zhidongx,
# ORGANIZATION:
#      VERSION: 4.0 for web site
#      CREATED: 08/16/2012 10:42:17 AM
#     REVISION: 2012.09.06
#===============================================================================

use lib '/home/zhidongx/test/bin/';
use strict;
use warnings;
use Getopt::Long;
use DomainHelper;

my $operating_system=$^O;
unless ($operating_system eq "linux"){
    printf("Your operating system $operating_system is unsupported at this time\n");
    printf("Currently only Linux is supported\n");
    exit();
}

my ($seqname,$datadir);
GetOptions('seqname:s' => \$seqname, 'datadir:s' => \$datadir);

if(!$seqname )
{
    printf "detectdcd USAGE:\n";
    printf "Mandatory arguments:\n";
    printf "./detectdcd.pl -seqname sequence_name\n";
    printf "Optional arguments whose default values have been set in the config.ini of this package.\n";
    printf "User can also replace the default values in the config file,like:\n";
    printf "      -datadir datadirectory: the Data_path item in the config.ini\n";
    
   exit();
}


my $helper       = new DomainHelper();
my %CONFIG       = $helper->readConfig();
my $dataset_path  = $CONFIG{'DATA_PATH'};  #where input/output files are

#########set domain parser enviroment  ###############



if($datadir){
    $dataset_path=$datadir;
}


my $line=`cat "$dataset_path/$seqname.result"`;
chop($line);
my $result = &checkdcd( $seqname,$dataset_path,$line );

`echo "$result" >"$dataset_path/$seqname.dcd"`;
printf "Discontinous domain detection:\n";
printf "$result\n";

sub checkdcd
{
    my ( $chainId, $base_path,$pred_domain) = @_;
#    print "--------------- $chainId---------------------------------\n";
    my $prediction_new = $pred_domain;
    open( RD2, "$base_path/$chainId.tpl" );
    my $numDCD      = 0;
    my $numGOOD     = 0;
    my $numTemplate = 0.000001;
    my ( %templates, %templates_GOOD );
    my %map;
    
    while ( my $ln = <RD2> )
    {
        $numTemplate++;
        $ln =~ /\S+\t\S+\t\S+\t(\d)\t(\d)\t(\S+)/;

        my $isDCD   = $1;
        my $isGOOD  = $2;
        my $domains = $3;
        if ( $isGOOD == 1 )
        {
            $templates_GOOD{$numGOOD} = $domains;
            $numGOOD++;
            if ( $isDCD == 1 )
            {
                $templates{$numDCD} = $domains;
                $numDCD++;
            }

        }
    }
    close(RD2);

    return $prediction_new  if($numGOOD<1 or $numTemplate<1);
    
    my $isDCD = 0;
    $pred_domain =~ /(\S+)\t(\S+)\t(\S+)\t(\S+)/;
    my $length   = $2;
    my $domainnum_pre=$3;
    my $domain   = $4;
    my @bs_pre   = &getBoundaries($domain);
    my %tmpmap;
    my $template;
    my @bs_tpl;
    if ( $numDCD / $numTemplate > 0.25 )
    {
        $isDCD = 1;
        my $maxedgenode = &cluster( \%templates, $numDCD,0.5 );
        $template = $templates{$maxedgenode};
        @bs_tpl = &getBoundaries($template);
        %tmpmap = &compareBoundaries( \@bs_pre, \@bs_tpl, 20 );
    }
    else
    {
        my $maxedgenode = &cluster( \%templates_GOOD, $numGOOD );
        $template = $templates_GOOD{$maxedgenode};
        @bs_tpl = &getBoundaries($template);
        %tmpmap = &compareBoundaries( \@bs_pre, \@bs_tpl, 20 );
    }
    if (  $isDCD==1 and $tmpmap{'identity'} >=0.4 
        or ($isDCD == 0 and $domainnum_pre>2 and $tmpmap{'identity'} >= 0.5 and $numGOOD>25))
    {
        my $prediction = &justifyBoundaries( \@bs_tpl, \@bs_pre, $template );
        my @tmp = split( /\)/, $prediction );
        my $domainnum = $#tmp + 1;
        $prediction_new = "$chainId\t$length\t$domainnum\t$prediction";
    }

    return $prediction_new;
}

sub cluster
{
    my ( $ref_map, $num ) = @_;
    my %clusterRecorder;
    my %templates = %$ref_map;
    for ( my $i = 0 ; $i < $num - 1 ; $i++ )
    {
        my $domain=$templates{$i};
        my @boundaries1 = &getBoundaries( $templates{$i} );
           
        for ( my $j = $i + 1 ; $j < $num ; $j++ )
        {
            my @boundaries2 = &getBoundaries( $templates{$j} );
            my %map = &compareBoundaries( \@boundaries1, \@boundaries2 );
            if ( $map{'num'} == 0 and $map{'identity'} > 0.9 )
            {
                $clusterRecorder{ $i, $j } = 1;
            }
            else
            {
                $clusterRecorder{ $i, $j } = 0;

            }
        }
    }

    for ( my $i = 0 ; $i < $num ; $i++ )
    {
        for ( my $j = 0 ; $j < $num ; $j++ )
        {
            if ( $i == $j )
            {
                $clusterRecorder{ $i, $j } = 1;
            }
            elsif ( not defined $clusterRecorder{ $i, $j } )
            {
                $clusterRecorder{ $i, $j } = $clusterRecorder{ $j, $i };

            }
        }
    }
    my @edges;
    for ( my $i = 0 ; $i < $num ; $i++ )
    {
        my $edge = 0;
        for ( my $j = 0 ; $j < $num ; $j++ )
        {

            #        print"$clusterRecorder{$i,$j} ";
            $edge += $clusterRecorder{ $i, $j };
        }
        push( @edges, $edge );

        #    print"\n";
    }

    #print "@edges\n";

    my $maxedgenum  = 0;
    my $maxedgenode = 0;
    for ( my $i = 0 ; $i <= $#edges ; $i++ )
    {
        if ( $maxedgenum < $edges[$i] )
        {
            $maxedgenum  = $edges[$i];
            $maxedgenode = $i;
        }
    }

    return $maxedgenode;
}

sub justifyBoundaries
{
    my ( $ref_tpl, $ref_pre, $domain_tpl ) = @_;

    my @array_tpl = @$ref_tpl;
    my @array_pre = @$ref_pre;

    #print "@array_tpl\n";
    #print "@array_pre\n";
    if ( $array_tpl[0] - $array_pre[0] > 40 )
    {
        my $temp = $array_tpl[0] - 1;
        $domain_tpl = "($array_pre[0]\-$temp)" . $domain_tpl;
    }else{
         $domain_tpl =~ s/\($array_tpl[0]\-/\(1\-/g;
    }
    if ( $array_pre[-1] - $array_tpl[-1] > 40 )
    {
        my $temp = $array_tpl[-1] + 1;
        $domain_tpl .= "($temp\-$array_pre[-1])";
    }else{

         $domain_tpl =~ s/\-$array_tpl[-1]\)/\-$array_tpl[-1]\)/g;
    }

    
    return $domain_tpl;
}

#===  FUNCTION  ================================================================
#         NAME: getBoundaries
#      PURPOSE:
#   PARAMETERS: ????
#      RETURNS: ????
#  DESCRIPTION: ????
#       THROWS: no exceptions
#     COMMENTS: none
#     SEE ALSO: n/a
#===============================================================================
sub getBoundaries
{
    my ($domain) = @_;
    my @boundaries;
    chop($domain);
    $domain =~ s/\(//g;
    my @array = split( /\)/, $domain );
    foreach my $dom (@array)
    {
        my @fragment = split( /\|/, $dom );
        foreach my $frag (@fragment)
        {
            $frag =~ /(\d+)-(\d+)/;
            push( @boundaries, $1 );
            push( @boundaries, $2 );
        }

    }
    @boundaries = sort { $a <=> $b } (@boundaries);
    return @boundaries;
}

#===  FUNCTION  ================================================================
#         NAME: compareBoundaries
#      PURPOSE:
#   PARAMETERS: array reference of two sorted domainboundaries;
#      RETURNS: map of comparison. key: num,minflag,maxflag,identity
#  DESCRIPTION: ????
#       THROWS: no exceptions
#     COMMENTS: none
#     SEE ALSO: n/a
#===============================================================================
sub compareBoundaries
{
    my ( $ref_1, $ref_2, $T ) = @_;
    my %map;

    $T = 5 if ( not $T );

    my @array1 = @$ref_1;
    my @array2 = @$ref_2;
    my $num1   = $#array1 + 1;
    my $num2   = $#array2 + 1;
    my $maxnum = $num1 > $num2 ? $num1 : $num2;
    my $minnum = $num1 < $num2 ? $num1 : $num2;

    if ( $maxnum == $minnum )
    {
        $map{'num'} = 0;
    }
    elsif ( $num1 == $maxnum )
    {
        $map{'num'} = 1;
    }
    else
    {
        $map{'num'} = -1;
    }

    #compare boundary position
    my @maxarray;
    my @minarray;
    if ( $map{'num'} >= 0 )
    {
        @maxarray = @array1;
        @minarray = @array2;
    }
    else
    {
        @maxarray = @array2;
        @minarray = @array2;
    }
    my $start = 0;
    my @minflag;
    my @maxflag;
    for ( my $i = 0 ; $i <= $#minarray ; $i++ )
    {
        for ( my $j = $start ; $j <= $#maxarray ; $j++ )
        {
            if ( abs( $minarray[$i] - $maxarray[$j] ) <= $T )
            {
                $start = $j + 1;
                push( @minflag, $i );
                push( @maxflag, $j );
                last;
            }
        }
    }

    $map{'minflag'}  = \@minflag;
    $map{'maxflag'}  = \@maxflag;
    $map{'identity'} = ( $#minflag + 1 ) / ( $#minarray + 1 );
    return %map;
}

